import unittest

from text_adventure import AdventureRoom


class AdventureRoomTestCase(unittest.TestCase):
    def test_instance(self):
        room = AdventureRoom('room1', 'desc room1')
        self.assertEqual(True, isinstance(room, AdventureRoom))

    def test_connect_rooms(self):
        room1 = AdventureRoom('room1', 'you\'re in a room1')
        room2 = AdventureRoom('room2', 'you\'re in a room2')

        room1.connect(direction='left', room=room2, description='there is a doorway to another room')
        self.assertEqual(1, len(room1.connections))
        self.assertEqual(1, len(room2.connections))

        self.assertEqual({'description': 'go back where you came from', 'room': room1}, room2.connections['back'])
        self.assertEqual({'description': 'there is a doorway to another room', 'room': room2}, room1.connections['left'])

    def test_options(self):
        room1 = AdventureRoom('room1', 'desc room1')
        room2 = AdventureRoom('room2', 'desc room2')

        room1.connect(direction='left', room=room2, description='you see a doorway to your left')
        self.assertEqual(['left'], room1.options)

    def test_run(self):
        room1 = AdventureRoom('room1', 'desc room1')
        room2 = AdventureRoom('room2', 'desc room2')

        room1.connect(direction='left', room=room2, description='you see a doorway to your left')
        self.assertEqual(['left'], room1.options)

    def test_string_representation(self):
        room1 = AdventureRoom('room1', 'desc room1')
        room2 = AdventureRoom('room2', 'desc room2')
        room1.connect(direction='left', room=room2, description='you see a doorway to your left')

        expected = '''
    Name:        "room1"
    Description: "desc room1"
    is_end:      "False"
	Connections: 
	 "left -> you see a doorway to your left"'''
        str_of = str(room1)
        self.assertEqual(expected, str_of)


if __name__ == '__main__':
    unittest.main()



class AdventureRoom:
    def __init__(self, name, description='', is_end=False):
        self.name = name
        self.description = description
        self.connections = {}
        self.is_end = is_end

    def connect(self, direction: str, room, description=''):
        if direction not in self.connections:
            self.connections[direction] = {'room': room, 'description': description}
            if direction != 'back':
                room.connect('back', self, description='go back where you came from')
        else:
            raise RuntimeError(f'there is already a room connected to that direction: {self.connections[direction]}')

    @property
    def options(self):
        return list(self.connections.keys())

    def get_direction_description(self, direction):
        if direction in self.connections:
            return self.connections[direction].get('description')

    def get_next_room(self, direction):
        if direction in self.connections:
            return self.connections[direction].get('room')

    def __str__(self):
        output = f'''
    Name:        "{self.name}"
    Description: "{self.description}"
    is_end:      "{self.is_end}"
'''
        if self.connections:
            output += "\tConnections: \n"
            for i in self.connections:
                output += f'\t "{i} -> {self.connections[i].get("description")}"'

        return output



import logging

from text_adventure.room import AdventureRoom


class TextAdventureEngine:
    def __init__(self, logger=None):
        self.rooms = {}
        self.starting_room = None

        if logger:
            self.logger = logger
        else:
            self.logger = logging.getLogger(__name__)

    def add_room(self, name, description, is_end=False):
        if name not in self.rooms:
            room = AdventureRoom(name, description, is_end)

            if 0 == len(self.rooms):
                self.starting_room = room

            self.rooms[name] = room
            return room
        else:
            raise RuntimeError('That room already exists')

    @property
    def number_rooms(self):
        return len(self.rooms)

    def _get_room_by_name(self, room_name):
        if room_name not in self.rooms:
            raise RuntimeError(f'Room named: "{room_name}" doesn\'t exist')
        else:
            return self.rooms[room_name]

    def connect_rooms(self, source: str, destination: str, direction: str, description=''):
        src_room = self._get_room_by_name(source)
        dst_room = self._get_room_by_name(destination)
        src_room.connect(direction=direction, room=dst_room, description=description)

    @staticmethod
    def _player_choice_to_direction(player_choice, current_room):
        try:
            idx = int(player_choice)
        except ValueError:
            return None

        if idx < len(current_room.options):
            direction = current_room.options[idx]
            return direction

    def start(self):
        current_room = self.starting_room

        while not current_room.is_end:
            idx = 0
            for direction in current_room.options:
                desc = current_room.get_direction_description(direction)
                print(f'{idx}: {direction} -> {desc}')
                idx += 1

            player_choice = input('Chose option: ')
            direction = self._player_choice_to_direction(player_choice, current_room)

            if direction is None:
                print('try again, with a valid choice this time please')
            else:
                self.logger.debug(f'you chose: {player_choice} -> {direction}')
                new_room = current_room.get_next_room(direction)
                current_room = new_room

        print(current_room.description)



import unittest
from unittest.mock import patch

from text_adventure import TextAdventureEngine


class TextAdventureEngineTestCase(unittest.TestCase):
    def test_instance(self):
        e = TextAdventureEngine()
        self.assertEqual(True, isinstance(e, TextAdventureEngine))

    def test_add_room(self):
        e = TextAdventureEngine()
        room1 = e.add_room(name='starting room', description='you wake up in a starting room')
        self.assertEqual(1, e.number_rooms)
        self.assertEqual('starting room', room1.name)

        room2 = e.add_room(name='room 1', description='room after leaving start')
        self.assertEqual(2, e.number_rooms)
        self.assertEqual('room 1', room2.name)

    def test_connect_rooms(self):
        e = TextAdventureEngine()
        room1 = e.add_room(name='starting room', description='you wake up in a starting room')
        room2 = e.add_room(name='room 1', description='room after leaving start')

        self.assertEqual(0, len(room1.connections))
        self.assertEqual(0, len(room2.connections))

        e.connect_rooms(source='starting room', destination='room 1', direction='left')
        self.assertEqual(1, len(room1.connections))
        self.assertEqual(1, len(room2.connections))

        self.assertEqual(room2, room1.connections['left'].get('room'))
        self.assertEqual(room1, room2.connections['back'].get('room'))

        with self.assertRaises(RuntimeError):
            e.connect_rooms(source='starting room', destination='doesnt exist', direction='right')

    @patch('builtins.input')
    @patch('builtins.print')
    def test_state_machine(self, mock_print, mock_input):

        e = TextAdventureEngine()
        e.add_room(name='starting room', description='you wake up in a starting room')
        e.add_room(name='room 1', description='room after leaving start')
        e.add_room(name='finish', description='outside world', is_end=True)

        e.connect_rooms(source='starting room', destination='room 1', direction='left', description='you see a door to your left')
        e.connect_rooms(source='room 1', destination='finish', direction='forward', description='the exit is right in front of you')

        # this will start interacting with the user, so we're going to mock out the user input and output
        mock_input.side_effect = ['0', '0', 'invalid', '0', 'blah', '1']
        e.start()

        self.assertTrue(mock_print.called)
        self.assertEqual(12, mock_print.call_count)


if __name__ == '__main__':
    unittest.main()
